@@ -1,3 +1,5 @@
+use http_02 as http;
+
 use std::{
     io::{Cursor, Read},
     net::{IpAddr, Ipv4Addr, SocketAddr},
@@ -5,16 +7,15 @@ use std::{
 
 use crate::{header::HeaderLine, response::ResponseStatusIndex, Request, Response};
 
-/// Converts an [http::Response] into a [Response](crate::Response).
+/// Converts an [`http::Response`] into a [`Response`].
 ///
-/// As an [http::Response] does not contain a URL, `"https://example.com/"` is
-/// used as a placeholder. Additionally, if the response has a header which
-/// cannot be converted into a valid [Header](crate::Header), it will be skipped
-/// rather than having the conversion fail. The remote address property will
-/// also always be `127.0.0.1:80` for similar reasons to the URL.
+/// As an [`http::Response`] does not contain a URL, `"https://example.com/"` is used as a
+/// placeholder. Additionally, if the response has a header which cannot be converted into
+/// the ureq equivalent, it will be skipped rather than having the conversion fail. The remote
+/// address property will also always be `127.0.0.1:80` for similar reasons to the URL.
 ///
-/// Requires feature `ureq = { version = "*", features = ["http"] }`
 /// ```
+/// # use http_02 as http;
 /// # fn main() -> Result<(), http::Error> {
 /// # ureq::is_test(true);
 /// let http_response = http::Response::builder().status(200).body("<response>")?;
@@ -38,12 +39,12 @@ impl<T: AsRef<[u8]> + Send + Sync + 'static> From<http::Response<T>> for Respons
             headers: value
                 .headers()
                 .iter()
-                .filter_map(|(name, value)| {
+                .map(|(name, value)| {
                     let mut raw_header: Vec<u8> = name.to_string().into_bytes();
-                    raw_header.extend([0x3a, 0x20]); // ": "
+                    raw_header.extend(b": ");
                     raw_header.extend(value.as_bytes());
 
-                    HeaderLine::from(raw_header).into_header().ok()
+                    HeaderLine::from(raw_header).into_header().unwrap()
                 })
                 .collect::<Vec<_>>(),
             reader: Box::new(Cursor::new(value.into_body())),
@@ -67,13 +68,8 @@ fn create_builder(response: &Response) -> http::response::Builder {
     let response_builder = response
         .headers
         .iter()
-        .filter_map(|header| {
-            header
-                .value()
-                .map(|safe_value| (header.name().to_owned(), safe_value.to_owned()))
-        })
         .fold(http::Response::builder(), |builder, header| {
-            builder.header(header.0, header.1)
+            builder.header(header.name(), header.value_raw())
         })
         .status(response.status())
         .version(http_version);
@@ -81,15 +77,11 @@ fn create_builder(response: &Response) -> http::response::Builder {
     response_builder
 }
 
-/// Converts a [Response](crate::Response) into an [http::Response], where the
-/// body is a reader containing the body of the response.
+/// Converts a [`Response`] into an [`http::Response`], where the body is a reader containing the
+/// body of the response.
 ///
-/// Due to slight differences in how headers are handled, this means if a header
-/// from a [Response](crate::Response) is not valid UTF-8, it will not be
-/// included in the resulting [http::Response].
-///
-/// Requires feature `ureq = { version = "*", features = ["http"] }`
 /// ```
+/// # use http_02 as http;
 /// # fn main() -> Result<(), ureq::Error> {
 /// # ureq::is_test(true);
 /// use std::io::Read;
@@ -104,15 +96,10 @@ impl From<Response> for http::Response<Box<dyn Read + Send + Sync + 'static>> {
     }
 }
 
-/// Converts a [Response](crate::Response) into an [http::Response], where the
-/// body is a String.
+/// Converts a [`Response`] into an [`http::Response`], where the body is a String.
 ///
-/// Due to slight differences in how headers are handled, this means if a header
-/// from a [Response](crate::Response) is not valid UTF-8, it will not be
-/// included in the resulting [http::Response].
-///
-/// Requires feature `ureq = { version = "*", features = ["http"] }`
 /// ```
+/// # use http_02 as http;
 /// # fn main() -> Result<(), ureq::Error> {
 /// # ureq::is_test(true);
 /// let response = ureq::get("http://example.com").call()?;
@@ -128,15 +115,10 @@ impl From<Response> for http::Response<String> {
     }
 }
 
-/// Converts a [Response](crate::Response) into an [http::Response], where the
-/// body is a Vec<u8>.
+/// Converts a [`Response`] into an [`http::Response`], where the body is a [`Vec<u8>`].
 ///
-/// Due to slight differences in how headers are handled, this means if a header
-/// from a [Response](crate::Response) is not valid UTF-8, it will not be
-/// included in the resulting [http::Response].
-///
-/// Requires feature `ureq = { version = "*", features = ["http"] }`
 /// ```
+/// # use http_02 as http;
 /// # fn main() -> Result<(), ureq::Error> {
 /// # ureq::is_test(true);
 /// let response = ureq::get("http://example.com").call()?;
@@ -146,21 +128,17 @@ impl From<Response> for http::Response<String> {
 /// ```
 impl From<Response> for http::Response<Vec<u8>> {
     fn from(value: Response) -> Self {
-        create_builder(&value)
-            .body(value.into_string().unwrap().into_bytes())
-            .unwrap()
+        let respone_builder = create_builder(&value);
+        let mut body_buf: Vec<u8> = vec![];
+        value.into_reader().read_to_end(&mut body_buf).unwrap();
+        respone_builder.body(body_buf).unwrap()
     }
 }
 
-/// Converts an [http] [Builder](http::request::Builder) into a [Request](crate::Request)
+/// Converts an [`http::request::Builder`] into a [`Request`].
 ///
-/// This will safely handle cases where a builder is not fully "complete" to
-/// prevent the conversion from failing. Should the requests' method or URI not
-/// be correctly set, the request will default to being a GET request to
-/// `"https://example.com"`. Additionally, any non-UTF8 headers will be skipped.
-///
-/// Requires feature `ureq = { version = "*", features = ["http"] }`
 /// ```
+/// # use http_02 as http;
 /// # fn main() -> Result<(), ureq::Error> {
 /// # ureq::is_test(true);
 /// let http_request_builder = http::Request::builder().method("GET").uri("http://example.com");
@@ -169,6 +147,46 @@ impl From<Response> for http::Response<Vec<u8>> {
 /// # Ok(())
 /// # }
 /// ```
+///
+/// # Fallibility
+///
+/// [`http::request::Builder`] contains a [`Result`] that is normally checked when the builder
+/// is "built" into a [`http::Request`].  This [`From`] implementation does _not_ check it
+/// however, and returns a `GET` [`Request`] that defaults to `"https://example.com"` in case
+/// it contains [`Err`].  In order to test for errors, utilize the provided conversion from
+/// [`http::request::Parts`]:
+///
+/// ```
+/// # use http_02 as http;
+/// # fn main() -> Result<(), ureq::Error> {
+/// ureq::is_test(true);
+/// let http_builder = http::Request::builder().method("GET").uri("http://example.com");
+/// let request = http_builder.body(()).expect("Builder error"); // Check the error
+/// let (parts, ()) = request.into_parts();
+/// let request: ureq::Request = parts.into();
+/// request.call()?;
+/// # Ok(())
+/// # }
+/// ```
+///
+/// # Converting from [`http::Request`]
+///
+/// Notably `ureq` does _not_ implement the conversion from [`http::Request`] because it contains
+/// the body of a request together with the actual request data.  However, [`http`] provides
+/// [`http::Request::into_parts()`] to split out a request into [`http::request::Parts`] and a
+/// `body`, for which the conversion _is_ implemented and can be used as follows:
+///
+/// ```
+/// # use http_02 as http;
+/// # fn main() -> Result<(), ureq::Error> {
+/// # ureq::is_test(true);
+/// let http_request = http::Request::builder().method("GET").uri("http://example.com").body(vec![0u8]).unwrap();
+/// let (http_parts, body) = http_request.into_parts();
+/// let request: ureq::Request = http_parts.into();
+/// request.send_bytes(&body)?;
+/// # Ok(())
+/// # }
+/// ```
 impl From<http::request::Builder> for Request {
     fn from(value: http::request::Builder) -> Self {
         let mut new_request = crate::agent().request(
@@ -179,32 +197,62 @@ impl From<http::request::Builder> for Request {
         );
 
         if let Some(headers) = value.headers_ref() {
-            new_request = headers
-                .iter()
-                .filter_map(|header| {
-                    header
-                        .1
-                        .to_str()
-                        .ok()
-                        .map(|str_value| (header.0.as_str(), str_value))
-                })
-                .fold(new_request, |request, header| {
-                    request.set(header.0, header.1)
-                });
+            for (name, value) in headers {
+                let mut raw_header: Vec<u8> = name.to_string().into_bytes();
+                raw_header.extend(b": ");
+                raw_header.extend(value.as_bytes());
+                let header = HeaderLine::from(raw_header).into_header().unwrap();
+
+                crate::header::add_header(&mut new_request.headers, header)
+            }
         }
 
         new_request
     }
 }
 
-/// Converts a [Request](crate::Request) into an [http] [Builder](http::request::Builder).
+/// Converts [`http::request::Parts`] into a [`Request`].
 ///
-/// This will only convert valid UTF-8 header values into headers on the
-/// resulting builder. The method and URI are preserved. The HTTP version will
-/// always be set to `HTTP/1.1`.
+/// An [`http::Request`] can be split out into its [`http::request::Parts`] and body as follows:
 ///
-/// Requires feature `ureq = { version = "*", features = ["http"] }`
 /// ```
+/// # use http_02 as http;
+/// # fn main() -> Result<(), ureq::Error> {
+/// # ureq::is_test(true);
+/// let http_request = http::Request::builder().method("GET").uri("http://example.com").body(vec![0u8]).unwrap();
+/// let (http_parts, body) = http_request.into_parts();
+/// let request: ureq::Request = http_parts.into();
+/// request.send_bytes(&body)?;
+/// # Ok(())
+/// # }
+/// ```
+impl From<http::request::Parts> for Request {
+    fn from(value: http::request::Parts) -> Self {
+        let mut new_request = crate::agent().request(value.method.as_str(), &value.uri.to_string());
+
+        for (name, value) in &value.headers {
+            // TODO: Aren't complete header values available as raw byte slices?
+            let mut raw_header: Vec<u8> = name.to_string().into_bytes();
+            raw_header.extend(b": ");
+            raw_header.extend(value.as_bytes());
+
+            let header = HeaderLine::from(raw_header)
+                .into_header()
+                .expect("Unreachable");
+
+            crate::header::add_header(&mut new_request.headers, header)
+        }
+
+        new_request
+    }
+}
+
+/// Converts a [`Request`] into an [`http::request::Builder`].
+///
+/// The method and URI are preserved. The HTTP version will always be set to `HTTP/1.1`.
+///
+/// ```
+/// # use http_02 as http;
 /// # fn main() -> Result<(), http::Error> {
 /// # ureq::is_test(true);
 /// let request = ureq::get("https://my-website.com");
@@ -219,13 +267,8 @@ impl From<Request> for http::request::Builder {
         value
             .headers
             .iter()
-            .filter_map(|header| {
-                header
-                    .value()
-                    .map(|safe_value| (header.name().to_owned(), safe_value.to_owned()))
-            })
             .fold(http::Request::builder(), |builder, header| {
-                builder.header(header.0, header.1)
+                builder.header(header.name(), header.value_raw())
             })
             .method(value.method())
             .version(http::Version::HTTP_11)
@@ -235,12 +278,14 @@ impl From<Request> for http::request::Builder {
 
 #[cfg(test)]
 mod tests {
+    use crate::header::{add_header, get_header_raw, HeaderLine};
+    use http_02 as http;
 
     #[test]
     fn convert_http_response() {
         use http::{Response, StatusCode, Version};
 
-        let http_response_body = (0..10240).into_iter().map(|_| 0xaa).collect::<Vec<u8>>();
+        let http_response_body = vec![0xaa; 10240];
         let http_response = Response::builder()
             .version(Version::HTTP_2)
             .header("Custom-Header", "custom value")
@@ -303,7 +348,7 @@ mod tests {
 
         let mut response = super::Response::new(418, "I'm a teapot", "some body text").unwrap();
         response.headers.push(
-            super::HeaderLine::from("Content-Type: text/plain".as_bytes().to_vec())
+            HeaderLine::from("Content-Type: text/plain".as_bytes().to_vec())
                 .into_header()
                 .unwrap(),
         );
@@ -327,15 +372,55 @@ mod tests {
     #[test]
     fn convert_to_http_response_bytes() {
         use http::Response;
-        use std::io::{Cursor, Read};
+        use std::io::Cursor;
 
         let mut response = super::Response::new(200, "OK", "tbr").unwrap();
-        response.reader = Box::new(Cursor::new(vec![0xde, 0xad, 0xbe, 0xef]));
-        let http_response: Response<Box<dyn Read + Send + Sync + 'static>> = response.into();
+        // b'\xFF' as invalid UTF-8 character
+        response.reader = Box::new(Cursor::new(vec![b'\xFF', 0xde, 0xad, 0xbe, 0xef]));
+        let http_response: Response<Vec<u8>> = response.into();
 
-        let mut buf = vec![];
-        http_response.into_body().read_to_end(&mut buf).unwrap();
-        assert_eq!(buf, vec![0xde, 0xad, 0xbe, 0xef]);
+        assert_eq!(
+            http_response.body().to_vec(),
+            // non UTF-8 byte is preserved
+            vec![b'\xFF', 0xde, 0xad, 0xbe, 0xef]
+        );
+    }
+
+    #[test]
+    fn convert_http_response_builder_with_invalid_utf8_header() {
+        use http::Response;
+
+        let http_response = Response::builder()
+            .header("Some-Key", b"some\xff\xffvalue".as_slice())
+            .body(b"hello")
+            .unwrap();
+        let response: super::Response = http_response.into();
+
+        assert_eq!(
+            get_header_raw(&response.headers, "some-key"),
+            Some(b"some\xff\xffvalue".as_slice())
+        );
+    }
+
+    #[test]
+    fn convert_to_http_response_builder_with_invalid_utf8_header() {
+        let mut response = super::Response::new(200, "OK", "tbr").unwrap();
+        add_header(
+            &mut response.headers,
+            HeaderLine::from(b"Some-Key: some\xff\xffvalue".to_vec())
+                .into_header()
+                .unwrap(),
+        );
+        dbg!(&response);
+        let http_response: http::Response<Vec<u8>> = response.into();
+
+        assert_eq!(
+            http_response
+                .headers()
+                .get("some-key")
+                .map(|h| h.as_bytes()),
+            Some(b"some\xff\xffvalue".as_slice())
+        );
     }
 
     #[test]
@@ -373,4 +458,45 @@ mod tests {
         assert_eq!(http_request.uri(), "http://some-website.com");
         assert_eq!(http_request.version(), http::Version::HTTP_11);
     }
+
+    #[test]
+    fn convert_http_request_builder_with_invalid_utf8_header() {
+        use http::Request;
+
+        let http_request = Request::builder()
+            .method("PUT")
+            .header("Some-Key", b"some\xff\xffvalue".as_slice())
+            .uri("https://google.com/?some=query");
+        let request: super::Request = http_request.into();
+
+        assert_eq!(
+            get_header_raw(&request.headers, "some-key"),
+            Some(b"some\xff\xffvalue".as_slice())
+        );
+        assert_eq!(request.method(), "PUT");
+        assert_eq!(request.url(), "https://google.com/?some=query");
+    }
+
+    #[test]
+    fn convert_to_http_request_builder_with_invalid_utf8_header() {
+        use http::request::Builder;
+
+        let mut request = crate::agent().head("http://some-website.com");
+        add_header(
+            &mut request.headers,
+            HeaderLine::from(b"Some-Key: some\xff\xffvalue".to_vec())
+                .into_header()
+                .unwrap(),
+        );
+        dbg!(&request);
+        let http_request_builder: Builder = request.into();
+        let http_request = http_request_builder.body(()).unwrap();
+
+        assert_eq!(
+            http_request.headers().get("some-key").map(|h| h.as_bytes()),
+            Some(b"some\xff\xffvalue".as_slice())
+        );
+        assert_eq!(http_request.uri(), "http://some-website.com");
+        assert_eq!(http_request.version(), http::Version::HTTP_11);
+    }
 }
