@@ -3,6 +3,7 @@ use std::io::{self, Write};
 use std::ops::Range;
 use std::time;
 
+use base64::{prelude::BASE64_STANDARD, Engine};
 use log::debug;
 use url::Url;
 
@@ -14,6 +15,7 @@ use crate::body::{self, BodySize, Payload, SizedReader};
 use crate::error::{Error, ErrorKind};
 use crate::header;
 use crate::header::{get_header, Header};
+use crate::proxy::Proto;
 use crate::resolve::ArcResolver;
 use crate::response::Response;
 use crate::stream::{self, connect_test, Stream};
@@ -87,7 +89,7 @@ impl Unit {
             if (!username.is_empty() || !password.is_empty())
                 && get_header(&headers, "authorization").is_none()
             {
-                let encoded = base64::encode(format!("{}:{}", username, password));
+                let encoded = BASE64_STANDARD.encode(format!("{}:{}", username, password));
                 extra.push(Header::new("Authorization", &format!("Basic {}", encoded)));
             }
 
@@ -215,9 +217,11 @@ pub(crate) fn connect(
         let mut headers = unit.headers;
 
         // on redirects we don't want to keep "content-length". we also might want to
-        // strip away "authorization" to ensure credentials are not leaked.
+        // strip away "authorization" and "cookie" to ensure credentials are not leaked.
         headers.retain(|h| {
-            !h.is_name("content-length") && (!h.is_name("authorization") || keep_auth_header)
+            !h.is_name("content-length")
+                && !h.is_name("cookie")
+                && (!h.is_name("authorization") || keep_auth_header)
         });
 
         // recreate the unit to get a new hostname and cookies for the new host.
@@ -403,12 +407,33 @@ fn send_prelude(unit: &Unit, stream: &mut Stream) -> io::Result<()> {
     // build into a buffer and send in one go.
     let mut prelude = PreludeBuilder::new();
 
+    let path = if let Some(proxy) = &unit.agent.config.proxy {
+        // HTTP proxies require the path to be in absolute URI form
+        // https://www.rfc-editor.org/rfc/rfc7230#section-5.3.2
+        match proxy.proto {
+            Proto::HTTP => match unit.url.port() {
+                Some(port) => format!(
+                    "{}://{}:{}{}",
+                    unit.url.scheme(),
+                    unit.url.host().unwrap(),
+                    port,
+                    unit.url.path()
+                ),
+                None => format!(
+                    "{}://{}{}",
+                    unit.url.scheme(),
+                    unit.url.host().unwrap(),
+                    unit.url.path()
+                ),
+            },
+            _ => unit.url.path().into(),
+        }
+    } else {
+        unit.url.path().into()
+    };
+
     // request line
-    prelude.write_request_line(
-        &unit.method,
-        unit.url.path(),
-        unit.url.query().unwrap_or_default(),
-    )?;
+    prelude.write_request_line(&unit.method, &path, unit.url.query().unwrap_or_default())?;
 
     // host header if not set by user.
     if !header::has_header(&unit.headers, "host") {
